/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2007  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "umtsmon_global.h"
#include "TheDeviceManagerSingleton.h"
#include "Device.h"
#include "TheSettingsSingleton.h"

#include <assert.h>

static TheDeviceManagerSingleton* theDeviceMgrSingletonInstancePtr = NULL;
static Device* theStaticDevicePtr = NULL;


static const char* thePPPKeyStr = "/device/PPPPortName";
static const char* theATKeyStr = "/device/ATPortName";
static const char* theDeviceTypeKeyStr = "/device/DeviceType";


TheDeviceManagerSingleton::TheDeviceManagerSingleton()
{
	DEBUG5("TheDeviceManagerSingleton::TheDeviceManagerSingleton()\n");

	// the non-autodetect is the first in the autodetect list
	// if anything gets supplied to this autodetector, it will attempt to
	// create the device
	// if that fails, the other autodetects will take over
	theNonAutoDetectPtr = new NonAutoDetect();
	theAutoDetectList.push_back(theNonAutoDetectPtr);

	// warning: make sure these remain in this order, because otherwise
	// you might get errors (e.g. the PCMCIA will error if it finds a USB2serial-based card)
	// and one definitely wants to use the USB autodetect over the RawUSBports autodetect 
	theAutoDetectList.push_back(new AutoDetect_USB());
	theAutoDetectList.push_back(new AutoDetect_PCMCIA());
	theAutoDetectList.push_back(new AutoDetect_CSSerialPorts());
	theAutoDetectList.push_back(new AutoDetect_PhoneModems());
	theAutoDetectList.push_back(new AutoDetect_RawUSBports());
//	theAutoDetectList.push_back(new AutoDetect_RawSerialPorts());
}

TheDeviceManagerSingleton::~TheDeviceManagerSingleton(void)
{
	DEBUG5("DESTRUCTOR TheDeviceManagerSingleton::~TheDeviceManagerSingleton()\n");
	
	// FIXME: delete autodetectlist contents here
}


// dear programmers, keep in mind that this member is static,
// i.e. you cannot use any of the other members in the class, 
// nor any of the variables 
TheDeviceManagerSingleton& TheDeviceManagerSingleton::me(void)
{
	// do we exist? if not: create us
	if (theDeviceMgrSingletonInstancePtr == NULL)
		theDeviceMgrSingletonInstancePtr = new TheDeviceManagerSingleton();
	return *theDeviceMgrSingletonInstancePtr;
}

// dear programmers, keep in mind that this member is static,
// i.e. you cannot use any of the other members in the class, 
// nor any of the variables 
DeviceCapabilities& TheDeviceManagerSingleton::getDeviceCapsRef(void)
{
	// do we exist? if not: create us
	if (theStaticDevicePtr == NULL)
		me().setupDevice();

	// at this moment, we are very sure that a DevicePtr exists,
	// even though it might be a null device...	
	// FIXME: No null device implemented yet!
	assert(theStaticDevicePtr != NULL);
	return dynamic_cast<DeviceCapabilities&>(*theStaticDevicePtr);
}

// dear programmers, keep in mind that this member is static,
// i.e. you cannot use any of the other members in the class, 
// nor any of the variables 
DeviceRadioCapabilities& TheDeviceManagerSingleton::getDeviceRadioCapsRef(void)
{
	// do we exist? if not: create us
	if (theStaticDevicePtr == NULL)
		me().setupDevice();

	// at this moment, we are very sure that a DevicePtr exists,
	// even though it might be a null device...	
	// FIXME: No null device implemented yet!
	assert(theStaticDevicePtr != NULL);
	return dynamic_cast<DeviceRadioCapabilities&>(*theStaticDevicePtr);
}

// dear programmers, keep in mind that this member is static,
// i.e. you cannot use any of the other members in the class, 
// nor any of the variables 
ConnectionInfo* TheDeviceManagerSingleton::getConnectionInfoPtr(void)
{
	// do we exist? if not: create us
	if (theDeviceMgrSingletonInstancePtr == NULL)
		theDeviceMgrSingletonInstancePtr = new TheDeviceManagerSingleton();

	// at this moment, we are very sure that a DevicePtr exists,
	// even though it might be a null device...	
	assert(theStaticDevicePtr != NULL);
	return theStaticDevicePtr->getConnectionInfoPtr();
}


bool
TheDeviceManagerSingleton::setupDevice(void)
{
	DEBUG3("TheDeviceManagerSingleton::setupDevice(void)\n");
	// did the user config on the command line?
	// did he force us not to look at the config file?
	if (theNonAutoDetectPtr->theSuggestedATPortName.isEmpty() 
	    && theNonAutoDetectPtr->theForceAutodetection==false)
	{
		readFromConfigFile();
	}

	// no? then let's run the autodetectors
	// note that the first autodetector is the NonAutoDetect,
	// that - if we have specified settings on command line or in 
	// the config file - will instantiate the Device immediately
	// without going through the autodetectors
	DEBUG3("Start iterating through all AutoDetectors\n\n");
	AutoDetectList::iterator myDetector = theAutoDetectList.begin();
	while (myDetector != theAutoDetectList.end())
	{
		if ((*myDetector)->go() == AutoDetectBase::DEVICE_DETECTED)
		{
			if (doDeviceCreation(**myDetector))
			{
				DEBUG3("Everything done - device created\n\n");
				return true;
			}
		}
		++myDetector;
		DEBUG3(" Let's continue with the next AutoDetector\n\n");
	}
	
	return false;
}


bool
TheDeviceManagerSingleton::doDeviceCreation(AutoDetectBase& aSuggestedAutoDetector)
{
	assert(theStaticDevicePtr==NULL);
	theStaticDevicePtr = aSuggestedAutoDetector.createDevice();
	if (theStaticDevicePtr==NULL)
		return false;
	
	// OK, so we got a real Device now. Let's store its settings for future use
	writeToConfigFile();
	return true;	
}


void TheDeviceManagerSingleton::readFromConfigFile(void)
{
	DEBUG3("TheDeviceManagerSingleton::readFromConfigFile()\n");
	theNonAutoDetectPtr->theSuggestedPPPPortName =
	TheSettingsSingleton::getQSRef().readEntry(thePPPKeyStr, ""); 

	theNonAutoDetectPtr->theSuggestedATPortName =
		TheSettingsSingleton::getQSRef().readEntry(theATKeyStr, "");
	
	theNonAutoDetectPtr->theSuggestedDeviceType = 
		static_cast<DeviceCapabilities::CardType>(
			TheSettingsSingleton::getQSRef().readNumEntry(theDeviceTypeKeyStr, 
			 	static_cast<int>(DeviceCapabilities::UNKNOWN)));	
}

void TheDeviceManagerSingleton::writeToConfigFile(void)
{
	DEBUG3("TheDeviceManagerSingleton::writeToConfigFile()\n");
	assert(theStaticDevicePtr != NULL);
	TheSettingsSingleton::getQSRef().writeEntry(thePPPKeyStr, 
			theStaticDevicePtr->getPPPSerialPortName() );	
	TheSettingsSingleton::getQSRef().writeEntry(theATKeyStr, 
			theStaticDevicePtr->getATSerialPortName() );	
	TheSettingsSingleton::getQSRef().writeEntry(theDeviceTypeKeyStr, 
			static_cast<int>(theStaticDevicePtr->getDeviceType()) );	
}

void
TheDeviceManagerSingleton::setSuggestedAT(const QString& anATPortName)
{
	assert(theStaticDevicePtr == NULL);
	assert(theNonAutoDetectPtr != NULL);
	theNonAutoDetectPtr->theSuggestedATPortName = anATPortName;
	DEBUG1("Set suggested AT port to '%s'\n", anATPortName.ascii()); 
}

void
TheDeviceManagerSingleton::setSuggestedPPP(const QString& aPPPPortName)
{
	assert(theStaticDevicePtr == NULL);
	assert(theNonAutoDetectPtr != NULL);
	theNonAutoDetectPtr->theSuggestedPPPPortName = aPPPPortName;
	DEBUG1("Set suggested PPP port to '%s'\n", aPPPPortName.ascii()); 
}

void
TheDeviceManagerSingleton::setSuggestedDeviceType(const DeviceCapabilities::CardType aDeviceType)
{
	assert(theStaticDevicePtr == NULL);
	assert(theNonAutoDetectPtr != NULL);
	theNonAutoDetectPtr->theSuggestedDeviceType = aDeviceType;
}

void
TheDeviceManagerSingleton::forceAutodetection(void)
{
	assert(theStaticDevicePtr == NULL);
	assert(theNonAutoDetectPtr != NULL);
	theNonAutoDetectPtr->theForceAutodetection = true;
	DEBUG1("Set force autodetection to true\n"); 
}
