/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "umtsmon_global.h"
#include "Device.h"
#include "Query.h"
#include "SerialPort.h"
#include "Popup.h"


Device::~Device()
{
	closeATSerial();
}


void
Device::closeATSerial(void)
{
	if (theATSerialPortPtr)
	{
		delete theATSerialPortPtr;
		theATSerialPortPtr = NULL;
		Query::setSerial(NULL);
	}
}



Device* 
Device::createDeviceInstance(CardType aCardType)
{
	Device* myDevicePtr = NULL;
	
	DEBUG3("Device creation for type 0x%08x requested\n", aCardType);
	switch (aCardType)
	{
	case UNKNOWN:
		// boo, bad, don't get here!!!
		// - will be handled outside the switch()
		DEBUG1("Attempt on creating device UNKNOWN - not good\n");
		break;
	case OPTION_3PORT: // is also for OPTION_4PORT
	case OPTION_NOZOMI:
		myDevicePtr = new Device_Option;
		break;
	case OPTION_VODAFONE_3G:
		myDevicePtr = new Device_Vodafone3G;
		break;
	case BROADCOM_GPRS_EDGE:
		myDevicePtr = new Device_BroadCom_GPRS_EDGE;
		break;
	case BROADCOM_57K6:
		myDevicePtr = new Device_BroadCom57k6;
		break;
	case HUAWEI_MULTI:
		myDevicePtr = new Device_Huawei;
		break;
	case GENERIC_USER_SERIAL:
		myDevicePtr = new Device_User_Serial;
		break;
	case GENERIC_CS_SERIAL:
		myDevicePtr = new Device_CS_Serial;
		break;
	case MODEM_ACM:
		myDevicePtr = new Device_ACM;
		break;
	case MODEM_RFCOMM:
		myDevicePtr = new Device_RFComm;
		break;
	}

	if (myDevicePtr != NULL)
		myDevicePtr->theDeviceType = aCardType;
	else
		Popup::Critical("Critical error in device setup\nPlease rerun with -v5\ne-mail results to author");
	return myDevicePtr;
}




bool
Device::probePortForAT( const QString& aPortName ) const
{
	DEBUG3("Device: probing port '%s' for AT commands\n", aPortName.ascii() );
	Query::ReturnCode myResult = Query::ERROR;


	SerialPort* mySerialPortPtr = new SerialPort;

	// the SerialPort class will automatically setup the Query class
	if (mySerialPortPtr->openDev(aPortName) == false)
	{
		goto not_good;
	}

	
	for (int i=0; i<3; i++)
	{
		Query myQueryAT("AT");
		myResult = myQueryAT.run();
 		if (myResult == Query::OK)
 		{
 			DEBUG3("Got Query::OK from port\n");
 			break;
 		}
 		if (myResult != Query::ERROR)
 		{
			DEBUG2("Device port '%s': no response to \"AT\", return code %d\n", 
					aPortName.ascii(), myResult);
			goto not_good;
 		}
 		// in case of ERROR: retry...
		DEBUG2("Device on port '%s' responded with ERROR, sleep & retry\n", 
			aPortName.ascii());
		sleep(1);
	}

not_good:
	// restore the original serial (whether it's NULL or not)
	Query::setSerial(theATSerialPortPtr);
	// and exit with returncode true or false
	DEBUG3("Device: probing port '%s' %s\n", aPortName.ascii(),
		(myResult == Query::OK)?"successful":"failed");
	return (myResult == Query::OK);
}

bool 
Device::setupPPPPort(const QString& aSerialPort)
{
	DEBUG5("setupPPPPort(%s)\n", aSerialPort.ascii());
	// the PPP name should be empty before we set it, as
	// this shouldn't need changing after setting it up once
	if (thePPPSerialPortName.isEmpty() != true)
		return false;
	
	// check if the port responds	
	if (probePortForAT(aSerialPort) == false)
		return false;
		
	thePPPSerialPortName = aSerialPort;
	
	return true;
}

// ###########################################################################
// ###########################################################################

Device_Option::Device_Option()
{
	theRadioPreferences = QStringList::split("#",
		"Only connect to GSM networks#"
		"Only connect to UMTS networks#"
		"If you have a choice - GPRS first#"
		"If you have a choice - UMTS first#"	
		"Stick to the network the card connects to#"
		"Automatic - let the card decide", false);
	// This list is taken from Paul Hardwick's "Option Fusion Howto" page.

	theConnectionInfoPtr = new ConnectionInfo; 
}


bool 
Device_Option::setupATPort(const QString& aSerialPort)
{
	if (probePortForAT(aSerialPort)==false)
		return false;

	theATSerialPortName = aSerialPort;
	theATSerialPortPtr = new SerialPort;

	// the SerialPort class will automatically setup the Query class
	if (theATSerialPortPtr->openDev(theATSerialPortName) == false)
	{
		closeATSerial();
		return false;
	}

	return true;
}

bool
Device_Option::setRadioPreferences(int aPref)
{
	if (aPref<0 || aPref>5)
		return false;

	QString myTempString;
	// FIXME: I have no clue what the second ,2 means...
	Query myQuery( "AT_OPSYS=" + myTempString.setNum(aPref) + ",2" );
	
 	Query::ReturnCode myResult = myQuery.runUntilDone();
 	if (myResult == Query::OK)
 		return true;
 	if (myResult == Query::NO_LOCK)
 		return false;
 		
/* FIXME: runUntilDone should render this code unnecessary
 	// There is a (rather big) chance that the card didn't respond in time. 
 	// Let's wait 100 us and retry
 	timespec myspec = { 0, 100*1000*1000 };
 	nanosleep( &myspec, NULL);
 	return (myQuery.receiveMore() == Query::OK);
*/
	return false;
} 


// ###########################################################################
// ###########################################################################


bool 
Device_Single_Serial::setupATPort(const QString& aSerialPort)
{
	if (probePortForAT(aSerialPort)==false)
		return false;

	theATSerialPortName = aSerialPort;
	theATSerialPortPtr = new SerialPort;

	// the SerialPort class will automatically setup the Query class
	if (theATSerialPortPtr->openDev(theATSerialPortName, getPortSpeed()) == false)
	{
		closeATSerial();
		return false;
	}

	return true;
}



bool 
Device_Single_Serial::setupPPPPort(const QString& aSerialPort)
{
	assert (thePPPSerialPortName.isEmpty());
	
	// this shouldn't be changeable after setting it up once
	if (thePPPSerialPortName.isEmpty() != true)
		return false;
	thePPPSerialPortName = aSerialPort;
	return true;
}


// ###########################################################################
// ###########################################################################

bool 
Device_ACM::setupATPort(const QString& aSerialPort)
{
	if (probePortForAT(aSerialPort)==false)
		return false;

	theATSerialPortName = aSerialPort;
	theATSerialPortPtr = new SerialPort;

	if (theATSerialPortPtr->openDev(theATSerialPortName, getPortSpeed()) == false)
	{
		closeATSerial();
		return false;
	}

	// use local scoping to ensure that this Query dies in time
	{
		// we need echo on, but for Moto phones, it's off by default.
		Query myQuery("ATE1");
		// don't use runUntilDone because the answer might not be complete... Sigh!!!
		DEBUG2("ACM: Result from ATE1 query: %d\n", myQuery.run());
	}
	
	return true;
}


// ###########################################################################
// ###########################################################################

bool 
Device_BroadCom_GPRS_EDGE::setupATPort(const QString& aSerialPort)
{
	if (Device_Single_Serial::setupATPort(aSerialPort) == false)
		return false;

	return doCFUN();
}

bool 
Device_BroadCom_GPRS_EDGE::doCFUN(void)
{
	Query myQueryAT("AT+CFUN=1,1");
	Query::ReturnCode myResult = myQueryAT.runUntilDone();
 	if (myResult != Query::OK)
		return false;
	return true;
}


// ###########################################################################
// ###########################################################################

bool 
Device_RFComm::setupATPort(const QString& aSerialPort)
{
	if (probePortForAT(aSerialPort)==false)
		return false;

	theATSerialPortName = aSerialPort;
	theATSerialPortPtr = new SerialPort;

	// the below line is the real reason why this class exists: a different timeout
	if (theATSerialPortPtr->openDev(theATSerialPortName, getPortSpeed(), 200000) == false)
	{
		closeATSerial();
		return false;
	}

	return true;
}


// ###########################################################################
// ###########################################################################

Device_Huawei::Device_Huawei() : Device_Option() 
{
	theRadioPrefsList.push_back( * new HuaweiRadioPrefs( 2,1, "GPRS/EDGE Preferred") );
	theRadioPrefsList.push_back( * new HuaweiRadioPrefs( 2,2, "3G Preferred") );
	theRadioPrefsList.push_back( * new HuaweiRadioPrefs(13,1, "GPRS/EDGE Only") );
	theRadioPrefsList.push_back( * new HuaweiRadioPrefs(14,2, "3G only") );

	// the Option constructor fills theRadioPreferences list wrongly.
	// let's make it right.
	theRadioPreferences.clear();
	RadioPrefsList::iterator myI = theRadioPrefsList.begin();
	while (myI != theRadioPrefsList.end())
	{
		theRadioPreferences.push_back( (*myI).theDescription);
		myI++;
	}

	theConnectionInfoPtr = new ConnectionInfo; 
}

bool
Device_Huawei::setRadioPreferences(int aPref)
{
	if (aPref<0 || aPref>3)
		return false;

	QString myDummy;
	QString myTempString = "AT^SYSCFG=" 
	         + myDummy.setNum(theRadioPrefsList[aPref].theFirst)
	         + "," 
	         + myDummy.setNum(theRadioPrefsList[aPref].theSecond)
	         + ",3FFFFFFF,1,2";

	Query myQuery( myTempString);
 	Query::ReturnCode myResult = myQuery.runUntilDone();
 	return (myResult == Query::OK);
} 
