/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEVICE_CLASS
#define DEVICE_CLASS


// forward declarations:
class SerialPort;

#include <assert.h>
#include <qstring.h>
#include <qstringlist.h>

#include <termios.h>
#include <vector>

#include "ConnectionInfo.h"
#include "DeviceCapabilities.h"


/// Device is an abstract base class that contains everything specific to a device
class Device : public DeviceCapabilities, public DeviceRadioCapabilities
{
public:
	// NOTICE THE MISSING CONSTRUCTOR HERE - IT'S IN THE PROTECTED SECTION

	/// destructor is public: anyone can delete this class (as opposed to constructing one)
	virtual ~Device();

	ConnectionInfo*  getConnectionInfoPtr(void)	
		{return theConnectionInfoPtr;};

	/** test if a port name is actually hosting a responding modem
	 *  @param aPortName	the name of the port to probe
	 *  @returns true if responsive
	 */
	bool probePortForAT( const QString& aPortName ) const;


	/** Factory method (see Design Patterns book) to create the Device class instance
	 *  The only way to create a Device instance is to use this method
	 *  @param	aCardType  - enum defines what Device class to construct
	 */
	static Device* createDeviceInstance(enum CardType aCardType);


public:
	/** This pure virtual function will actually initialise everything that can fail
	 *  @param  aSerialPort	name of the serial port that was detected (or forced)
	 *  @returns 			true if everything was OK
	 */
	virtual bool setupATPort(const QString& aSerialPort) = 0;

	/// closes and deletes theATSerialPortName
	void closeATSerial(void);

	/** setupPPPPort will set the port to be used to setup PPP connections
	 *  if this is not done before calling startPPP, this will result in trouble
	 *  @param aSerialPort	device name of the serial port to use for PPP commands
	 *  @returns	true if setup was successful
	 */
	virtual bool setupPPPPort(const QString& aSerialPort);


	/** answers if the modem has a method to set radio preferences (UMTS only, GPRS only, etc)
	 *  @returns  true if the Device class has an implementation for this
	 */
	virtual bool hasRadioPreferences(void) const  {return false;};
	
	/// returns a list of all Radio Preferences the card supports, default setting is first 
	virtual const QStringList getRadioPreferences(void)    {return NULL;};
	
	/// returns true if setting the Radio Preference was OK
	virtual bool setRadioPreferences( __attribute__ ((unused)) int aPref)   {return false;}; 

	/// returns the default radio preference for the card
	virtual int getDefaultRadioPreference(void)   {return 0;}; 

	/// returns a string describing the Device C++ class - not the device itself
	virtual QString getDeviceClassName(void) = 0;


	const QString&	getPPPSerialPortName(void) const	
		{ return thePPPSerialPortName; };

	const QString&	getATSerialPortName(void) const	
		{ return theATSerialPortName; };


	/// returns the type of the Device
	virtual const DeviceCapabilities::CardType getDeviceType(void) const
		{ return theDeviceType; };

protected:
	QString		theATSerialPortName;
	QString		thePPPSerialPortName;

	ConnectionInfo* theConnectionInfoPtr;
	SerialPort*	theATSerialPortPtr;
	
	CardType	theDeviceType;

protected:
	/** Protected constructor: this class cannot be constructed by 3rdParties.
	 *  call the *static* createDeviceInstance method for that
	 */
	Device() : theConnectionInfoPtr(NULL),
		   theATSerialPortPtr(NULL) {};

private:
	/// private, unimplemented copy constructor - don't use
	Device(const Device&);
	/// private, unimplemented assign operator - don't use
	Device& operator=(Device& aPtr);
};


//############################################################################
//############################################################################
//############################################################################

/// implementation of Device class for Option cards (both GT and Fusion+ cards)
class Device_Option : public Device
{
public:
	Device_Option();
	virtual ~Device_Option () {};

	/** implemenation of pure virtual function will actually initialise everything that can fail
	 *  @param  aSerialPort		name of the serial port that was detected (or forced)
	 *  @returns 			bool if everything was OK
	 */
	virtual bool setupATPort(const QString& aSerialPort);

	/// The Option cards have AT_OPSYS to set radio preferences
	virtual bool hasRadioPreferences(void) const  {return true;};
	
	/// returns a list of all Radio Preferences the card supports, default setting is first 
	virtual const QStringList getRadioPreferences(void) {return theRadioPreferences;};

	/// returns true if setting the Radio Preference was OK
	virtual bool setRadioPreferences(int  aPref); 

	/// the default radio preference for Option Cards is "prefer UMTS"
	virtual int getDefaultRadioPreference(void)   {return 3;}; 

	virtual int  getNumberOfPorts(void) const {return 3;};

	virtual QString getDeviceClassName(void) {return "Qualcomm-based chipset\n"; };

protected:
	QStringList theRadioPreferences;
};


//############################################################################
//############################################################################
//############################################################################

/** implementation of Device class for Huawei cards with multiple ports
 *  (as far as Klaas knows, all cards by Huawei qualify - at least E220/E612/E620)
 */
class Device_Huawei : public Device_Option
{
public:
	Device_Huawei(); 
	virtual ~Device_Huawei () {};

	virtual bool hasRadioPreferences(void) const  {return true;};
	virtual bool setRadioPreferences(int  aPref); 
	virtual int getDefaultRadioPreference(void)   {return 1;}; /// 3G preferred 

	virtual int  getNumberOfPorts(void) const {return 2;};
	virtual QString getDeviceClassName(void) {return "Generic Huawei card\n"; };

private:
	struct HuaweiRadioPrefs
	{
		int     theFirst;
		int     theSecond;
		QString theDescription;
		
		HuaweiRadioPrefs(int a, int b, const QString& c) 
			: theFirst(a), theSecond(b), theDescription(c) {};
	};
	
	typedef std::vector<HuaweiRadioPrefs> RadioPrefsList;
	RadioPrefsList theRadioPrefsList;
	
};

//############################################################################
//############################################################################
//############################################################################

/** implementation of Device class for the original Vodafone 3G card
 *  different in usb hub IDs, has a known bug in PIN handling
 */
class Device_Vodafone3G : public Device_Option
{
public:
	Device_Vodafone3G() { theConnectionInfoPtr = new ConnectionInfo; };
	virtual ~Device_Vodafone3G () {};

	virtual int  getNumberOfPorts(void) const {return 3;};
	virtual QString getDeviceClassName(void) {return "Original Vodafone 3G card\n"; };
};


//############################################################################
//############################################################################
//############################################################################

/** the Single Serial Device abstracts the notion of everything that provides
 *  only one Serial Port - this in contrast to e.g. the Open 4 port usb2serial and the Nozomi
 *  devices which are 3 or 4 ports.
 * 
 *  A Single Serial device has concurrency issues - whilst a PPP connection is open,
 *  it is not possible (for now) to do e.g. signal quality requests
 */
class Device_Single_Serial : public Device
{
public:
	Device_Single_Serial() { theConnectionInfoPtr = new ConnectionInfo; };
	virtual ~Device_Single_Serial()	{};

	/** This pure virtual function will actually initialise everything that can fail
	 *  @param  aSerialPort	name of the serial port that was detected (or forced)
	 *  @returns 			true if everything was OK
	 */
	virtual bool setupATPort(const QString& aSerialPort);

	/** setupPPPPort will set the port to be used to setup PPP connections
	 *  if this is not done before calling startPPP, this will result in trouble
	 *  @param aSerialPort	device name of the serial port to use for PPP commands
	 *  @returns	true if setup was successful
	 */
	virtual bool setupPPPPort(const QString& aSerialPort);

	virtual int  getNumberOfPorts(void) const {return 1;};
	
protected:
	/// returns the port speed (as a termios number, e.g. B115200) of the single serial port
	virtual int  getPortSpeed(void) = 0;
};


//############################################################################
//############################################################################
//############################################################################

/// implemention of Device class for Broadcom 4306-based cards (Sony Ericsson GC79, Option GT EDGE and GT Combo EDGE)
class Device_BroadCom_GPRS_EDGE : public Device_Single_Serial
{
public:
	virtual ~Device_BroadCom_GPRS_EDGE() {};

	/** implemenation of pure virtual function will actually initialise everything that can fail
	 *  @param  aSerialPort		name of the serial port that was detected (or forced)
	 *  @returns 			bool if everything was OK
	 */
	virtual bool setupATPort(const QString& aSerialPort);

	virtual QString getDeviceClassName(void) {return "Broadcom based 2G card\n"; };

protected:
	/** this function needs to be called on start of the device to power on
	 *  the radio.
	 */
	bool doCFUN(void);

	/// returns the port speed (as a termios number, e.g. B115200) of the single serial port
	virtual int  getPortSpeed(void) {return (B115200);};
};

//############################################################################
class Device_BroadCom57k6 : public Device_BroadCom_GPRS_EDGE 
{
public:
	virtual QString getDeviceClassName(void) {return "Broadcom based 2.5G card\n"; };

protected:
	/// returns the port speed (as a termios number, e.g. B115200) of the single serial port
	virtual int  getPortSpeed(void) {return (B57600);};
};


//############################################################################
//############################################################################
//############################################################################

/** implemention of Device class that handles a user-configured serial port.
 *  don't expect this Device class to contain much functionality
 */ 
class Device_User_Serial : public Device_Single_Serial
{
public:
	virtual ~Device_User_Serial() {};

	virtual QString getDeviceClassName(void) {return "User provided serial port\n"; };

protected:
	/// returns the port speed (as a termios number, e.g. B115200) of the single serial port
	int  getPortSpeed(void) {return (B115200);};
};


//############################################################################
//############################################################################
//############################################################################

/** implemention of Device class that handles a PCNCIA cs_serial devices
 *  this cs_serial device does not show up in the pcmcia devices list.
 */ 
class Device_CS_Serial : public Device_Single_Serial
{
public:
	virtual ~Device_CS_Serial() {};

	virtual QString getDeviceClassName(void) {return "Generic PCMCIA serial modem\n"; };

protected:
	/// returns the port speed (as a termios number, e.g. B115200) of the single serial port
	int  getPortSpeed(void) {return (B115200);};
};



//############################################################################
//############################################################################
//############################################################################

/// USB-connected mobile phone modem - e.g. connected over /dev/ttyACM0
class Device_ACM : public Device_Single_Serial
{
public:
	virtual ~Device_ACM() {};

	virtual QString getDeviceClassName(void) {return "USB-connected mobile phone modem\n"; };

	/** implemenation of pure virtual function will actually initialise everything that can fail
	 *  @param  aSerialPort		name of the serial port that was detected (or forced)
	 *  @returns 			bool if everything was OK
	 */
	bool setupATPort(const QString& aSerialPort);
	
protected:
	/// returns the port speed (as a termios number, e.g. B115200) of the single serial port
	int  getPortSpeed(void) {return (B115200);};
};


//############################################################################
//############################################################################
//############################################################################

/** Bluetooth connected mobile phone modem - e.g. connected over /dev/rfcomm0
 * 
 * This type of modem is special because it requires modifications for the 
 * roundtrip latencies in the serial connection.
 */ 
class Device_RFComm : public Device_Single_Serial
{
public:
	virtual ~Device_RFComm() {};

	virtual QString getDeviceClassName(void) {return "Bluetooth connected mobile phone modem\n"; };

	/** implemenation of pure virtual function will actually initialise everything that can fail
	 *  @param  aSerialPort		name of the serial port that was detected (or forced)
	 *  @returns 			bool if everything was OK
	 */
	virtual bool setupATPort(const QString& aSerialPort);

protected:
	/// returns the port speed (as a termios number, e.g. B115200) of the single serial port
	int  getPortSpeed(void) {return (B57600);};
};



#endif // defined DEVICE_CLASS
